@@ -1,4 +1,4 @@
-# Copyright (c) 2020, Djaodjin Inc.
+# Copyright (c) 2021, Djaodjin Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -25,11 +25,14 @@ import logging
 
 from django.http import Http404
 from django.core.exceptions import ValidationError
+from django.db import transaction
 from django.db.models import Q
-from rest_framework.mixins import CreateModelMixin
 from rest_framework import generics, response as api_response
+from rest_framework.mixins import CreateModelMixin
+from rest_framework.filters import SearchFilter
 
-from ..models import PageElement, build_content_tree
+from .. import settings
+from ..models import PageElement, build_content_tree, flatten_content_tree
 from ..serializers import (NodeElementSerializer, PageElementSerializer,
     PageElementTagSerializer)
 from ..mixins import AccountMixin, PageElementMixin, TrailMixin
@@ -41,7 +44,7 @@ LOGGER = logging.getLogger(__name__)
 
 class PageElementSearchAPIView(AccountMixin, generics.ListAPIView):
     """
-    Search through editable page elements
+    Search through page elements
 
     **Tags: content
 
@@ -75,7 +78,7 @@ class PageElementSearchAPIView(AccountMixin, generics.ListAPIView):
             if search_string is not None:
                 validate_title(search_string)
                 queryset = queryset.filter(
-                    Q(tag__icontains=search_string)
+                    Q(extra__icontains=search_string)
                     | Q(title__icontains=search_string))
                 return queryset
         except ValidationError:
@@ -83,9 +86,9 @@ class PageElementSearchAPIView(AccountMixin, generics.ListAPIView):
         return []
 
 
-class PageElementTreeAPIView(TrailMixin, generics.RetrieveAPIView):
+class PageElementAPIView(TrailMixin, generics.ListAPIView):
     """
-    Retrieves a content tree
+    Lists a tree of page elements
 
     **Tags: content
 
@@ -93,59 +96,185 @@ class PageElementTreeAPIView(TrailMixin, generics.RetrieveAPIView):
 
     .. code-block:: http
 
-        GET /api/content HTTP/1.1
+        GET /api/content/boxes-and-enclosures HTTP/1.1
 
     responds
 
     .. code-block:: json
 
         {
-            "slug": "boxes-enclosures",
-            "path": "/boxes-enclosures",
-            "title": "Content root",
+          "count": 8,
+          "next": null,
+          "previous": null,
+          "results": [
+          {
+            "slug": "metal",
+            "path": null,
+            "title": "Metal structures & equipment",
+            "indent": 0
+          },
+          {
+            "slug": "boxes-and-enclosures",
+            "path": "/metal/boxes-and-enclosures",
+            "title": "Boxes & enclosures",
+            "indent": 1,
+            "tags": [
+              "industry",
+              "pagebreak",
+              "public",
+              "scorecard"
+            ]
+          }
+          ]
+        }
+    """
+    serializer_class = PageElementSerializer
+    queryset = PageElement.objects.all()
+
+    def attach(self, elements):
+        return elements
+
+    def list(self, request, *args, **kwargs):
+        #pylint:disable=unused-argument
+        content_tree = build_content_tree(
+            roots=[self.element] if self.element else None,
+            prefix=self.full_path)
+        results = flatten_content_tree(content_tree)
+        self.attach(results)
+
+        results.pop(0)
+        self.element.path = self.full_path
+        self.element.results = results
+        self.element.count = len(results)
+        serializer = self.get_serializer(self.element)
+        return api_response.Response(serializer.data)
+
+
+class PageElementDetailAPIView(AccountMixin, PageElementMixin,
+                               generics.RetrieveAPIView):
+    """
+    Retrieves details on a page element
+
+    **Tags: content
+
+    **Example
+
+    .. code-block:: http
+
+        GET /api/content/detail/adjust-air-fuel-ratio HTTP/1.1
+
+    responds
+
+    .. code-block:: json
+
+        {
+            "slug": "adjust-air-fuel-ratio",
             "picture": null,
-            "extra": null,
-            "children": []
+            "title": "Adjust air/fuel ratio",
+            "text": "<h2>Background</h2><p>Some manufacturing processes may\
+ involve heating operations.</p>",
+            "extra": null
+        }
+    """
+    serializer_class = PageElementSerializer
+
+    def get_object(self):
+        return self.element
+
+
+class PageElementEditableListAPIView(TrailMixin, generics.ListCreateAPIView):
+    """
+    List editable page elements
+
+    This API endpoint lists page elements that are owned and thus editable
+    by an account.
+
+    **Tags**: editors
+
+    **Examples
+
+    .. code-block:: http
+
+        GET /api/content/editables/energy-utility/ HTTP/1.1
+
+    responds
+
+    .. code-block:: json
+
+        {
+            "count": 5,
+            "next": null,
+            "previous": null,
+            "results": [
+                {
+                    "path": null,
+                    "title": "Construction",
+                    "tags": ["public"],
+                    "indent": 0
+                },
+                {
+                    "path": null,
+                    "title": "Governance & management",
+                    "picture": "https://assets.tspproject.org/management.png",
+                    "indent": 1
+                },
+                {
+                    "path": "/construction/governance/the-assessment\
+-process-is-rigorous",
+                    "title": "The assessment process is rigorous",
+                    "indent": 2,
+                    "environmental_value": 1,
+                    "business_value": 1,
+                    "profitability": 1,
+                    "implementation_ease": 1,
+                    "avg_value": 1
+                },
+                {
+                    "path": null,
+                    "title": "Production",
+                    "picture": "https://assets.tspproject.org/production.png",
+                    "indent": 1
+                },
+                {
+                    "path": "/construction/production/adjust-air-fuel\
+-ratio",
+                    "title": "Adjust Air fuel ratio",
+                    "indent": 2,
+                    "environmental_value": 2,
+                    "business_value": 2,
+                    "profitability": 2,
+                    "implementation_ease": 2,
+                    "avg_value": 2
+                }
+            ]
         }
     """
     serializer_class = NodeElementSerializer
-    queryset = PageElement.objects.all()
+    filter_backends = (SearchFilter,)
 
-    def retrieve(self, request, *args, **kwargs):
-        path_parts = self.get_full_element_path(self.path)
-        roots = path_parts[-1] if path_parts else None
-        content_tree = build_content_tree(roots=roots, prefix=self.path)
-        self.attach_picture(content_tree, self.get_pictures())
-        return api_response.Response(content_tree)
-
-    def get_roots(self):
-        # XXX return self.get_queryset().get_roots()
-        # XXX exception `AttributeError: 'QuerySet' object has no attribute
-        # XXX 'get_roots'`
-        return PageElement.objects.get_roots()
-
-    def get_pictures(self):
-        results = {}
-        for item in self.get_queryset().filter(
-            text__endswith='.png').values('slug', 'text'):
-            results.update({item['slug']: item['text']})
-        return results
-
-    def attach_picture(self, content_tree, pictures, prefix_picture=None):
-        for path, node in content_tree.items():
-            slug = path.split('/')[-1]
-            prefix_picture = pictures.get(slug, prefix_picture)
-            node[0].update({'picture': prefix_picture})
-            self.attach_picture(
-                node[1], pictures, prefix_picture=prefix_picture)
+    def get_queryset(self):
+        """
+        Returns a list of heading and best practices
+        """
+        account_url_kwarg = settings.ACCOUNT_URL_KWARG
+        if account_url_kwarg in self.kwargs:
+            queryset = PageElement.objects.filter(account=self.account)
+        else:
+            queryset = PageElement.objects.all()
+        if self.path:
+            queryset = queryset.filter(consumption__path__startswith=self.path)
+#        queryset = queryset.annotate(
+#            nb_referencing_practices=Count('consumption')).order_by('title')
+        return queryset
 
 
-class PageElementDetail(PageElementMixin, CreateModelMixin,
-                        generics.RetrieveUpdateDestroyAPIView):
+class PageElementEditableDetail(AccountMixin, PageElementMixin,
+                                CreateModelMixin,
+                                generics.RetrieveUpdateDestroyAPIView):
     """
-    Retrieves details on an editable page element
+    Retrieves an editable page element
 
-    **Tags: content
+    **Tags: editors
 
     **Example
 
@@ -165,11 +294,14 @@ class PageElementDetail(PageElementMixin, CreateModelMixin,
     """
     serializer_class = PageElementSerializer
 
+    def get_object(self):
+        return self.element
+
     def delete(self, request, *args, **kwargs):
         """
-        Deletes an editable page element
+        Deletes a page element
 
-        **Tags: content
+        **Tags: editors
 
         **Example
 
@@ -178,13 +310,14 @@ class PageElementDetail(PageElementMixin, CreateModelMixin,
             DELETE /api/content/editables/boxes-enclosures/ HTTP/1.1
         """
         #pylint:disable=useless-super-delegation
-        return super(PageElementDetail, self).delete(request, *args, **kwargs)
+        return super(PageElementEditableDetail, self).delete(
+            request, *args, **kwargs)
 
     def post(self, request, *args, **kwargs):
         """
-        Creates an editable page element
+        Creates a page element
 
-        **Tags: content
+        **Tags: editors
 
         **Example
 
@@ -209,11 +342,14 @@ class PageElementDetail(PageElementMixin, CreateModelMixin,
 
         """
         #pylint:disable=useless-super-delegation
-        return super(PageElementDetail, self).create(request, *args, **kwargs)
+        return super(PageElementEditableDetail, self).create(
+            request, *args, **kwargs)
 
     def put(self, request, *args, **kwargs):
         """
-        Updates an editable page element
+        Updates a page element
+
+        **Tags: editors
 
         **Example
 
@@ -237,29 +373,31 @@ class PageElementDetail(PageElementMixin, CreateModelMixin,
             }
         """
         #pylint:disable=useless-super-delegation
-        return super(PageElementDetail, self).put(request, *args, **kwargs)
+        return super(PageElementEditableDetail, self).put(
+            request, *args, **kwargs)
 
     def perform_create(self, serializer):
         serializer.save(account=self.account)
 
     def update(self, request, *args, **kwargs):
-        try:
-            _ = self.get_object()
-            return super(PageElementDetail, self).update(
+        with transaction.atomic():
+            try:
+                _ = self.element
+                return super(PageElementEditableDetail, self).update(
+                    request, *args, **kwargs)
+            except Http404:
+                pass
+            return super(PageElementEditableDetail, self).create(
                 request, *args, **kwargs)
-        except Http404:
-            pass
-        return super(PageElementDetail, self).create(request, *args, **kwargs)
 
 
-class PageElementAddTags(PageElementMixin, generics.UpdateAPIView):
+class PageElementAddTags(AccountMixin, PageElementMixin,
+                         generics.UpdateAPIView):
     """
     Adds tags to an editable node
 
     Add tags to a ``PageElement`` if they are not already present.
 
-    **Tags: content
-
     **Example
 
     .. code-block:: http
@@ -281,8 +419,11 @@ class PageElementAddTags(PageElementMixin, generics.UpdateAPIView):
     """
     serializer_class = PageElementTagSerializer
 
+    def get_object(self):
+        return self.element
+
     def perform_update(self, serializer):
-        curr_tags = serializer.instance.tag
+        curr_tags = serializer.instance.extra
         if curr_tags:
             curr_tags = curr_tags.split(',')
         else:
@@ -291,18 +432,17 @@ class PageElementAddTags(PageElementMixin, generics.UpdateAPIView):
         for tag in add_tags:
             if not tag in curr_tags:
                 curr_tags.append(tag)
-        serializer.instance.tag = ','.join(curr_tags)
+        serializer.instance.extra = ','.join(curr_tags)
         serializer.instance.save()
 
 
-class PageElementRemoveTags(PageElementMixin, generics.UpdateAPIView):
+class PageElementRemoveTags(AccountMixin, PageElementMixin,
+                            generics.UpdateAPIView):
     """
     Remove tags from an editable node
 
     Remove tags from a ``PageElement``.
 
-    **Tags: content
-
     **Examples
 
     .. code-block:: http
@@ -326,8 +466,11 @@ class PageElementRemoveTags(PageElementMixin, generics.UpdateAPIView):
     """
     serializer_class = PageElementTagSerializer
 
+    def get_object(self):
+        return self.element
+
     def perform_update(self, serializer):
-        curr_tags = serializer.instance.tag
+        curr_tags = serializer.instance.extra
         if curr_tags:
             curr_tags = curr_tags.split(',')
         else:
@@ -336,5 +479,5 @@ class PageElementRemoveTags(PageElementMixin, generics.UpdateAPIView):
         for tag in remove_tags:
             if tag in curr_tags:
                 curr_tags.remove(tag)
-        serializer.instance.tag = ','.join(curr_tags)
+        serializer.instance.extra = ','.join(curr_tags)
         serializer.instance.save()
