@@ -1,4 +1,4 @@
-# Copyright (c) 2020, Djaodjin Inc.
+# Copyright (c) 2021, Djaodjin Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -25,7 +25,9 @@
 
 import hashlib, os
 
+from django.db import transaction
 from django.utils.encoding import force_text
+from django.utils.translation import ugettext_lazy as _
 from deployutils.helpers import datetime_or_now
 from rest_framework import parsers, status
 from rest_framework.generics import ListCreateAPIView
@@ -86,16 +88,16 @@ class MediaListAPIView(UploadedImageMixin, AccountMixin, ListCreateAPIView):
             prefix=kwargs.get('path', '.'))
         return self.get_paginated_response(results)
 
-    def get_paginated_response(self, results):
+    def get_paginated_response(self, data):
         # XXX - Deactivate pagination until not
         # implemented in djaodjin-sidebar-gallery
         # page = self.paginate_queryset(queryset['results'])
         # if page is not None:
         #     queryset = {'count': len(page), 'results' : page}
-        total_count = len(results)
+        total_count = len(data)
         return Response({
             'count': total_count,
-            'results': sorted(results, key=lambda x: x['updated_at'])
+            'results': sorted(data, key=lambda x: x['updated_at'])
         })
 
     def post(self, request, *args, **kwargs):
@@ -156,42 +158,34 @@ class MediaListAPIView(UploadedImageMixin, AccountMixin, ListCreateAPIView):
 
         .. code-block:: http
 
-            DELETE /api/assets/ HTTP/1.1
-
-        .. code-block:: json
-
-            {
-                "items": [
-                    {"location": "/media/item/url1.jpg"},
-                    {"location": "/media/item/url2.jpg"}
-                ]
-            }
+            DELETE /api/assets/?location=/media/item/url1.jpg HTTP/1.1
 
         """
         #pylint: disable=unused-argument,too-many-locals
         storage = get_default_storage(self.request, self.account)
-        serializer = MediaItemListSerializer(data=request.data)
-        serializer.is_valid()
-        validated_data = serializer.validated_data
-        filter_list = self.build_filter_list(validated_data)
-        results, _ = self.list_media(storage, filter_list,
-            prefix=kwargs.get('path', '.'))
-        if not results:
+        assets, total_count = self.list_media(
+            storage,
+            self.build_filter_list({'items': [
+                {'location': request.query_params.get('location')}]}))
+        if not assets:
             return Response({}, status=status.HTTP_404_NOT_FOUND)
 
         base = storage.url('')
-        for item in results:
-            location = item['location']
+        for item in assets:
+            parts = urlparse(item['location'])
+            location = urlunparse((parts.scheme, parts.netloc, parts.path,
+                None, None, None))
             if location.startswith(base):
-                location = location[len(base):]
-                storage.delete(location)
-                # Delete all MediaTag and PageElement using this location
-                MediaTag.objects.filter(location=location).delete()
-                elements = PageElement.objects.filter(text=location)
-                for element in elements:
-                    element.text = ""
-                    element.save()
-        return Response({}, status=status.HTTP_204_NO_CONTENT)
+                storage.delete(location[len(base):])
+            # Delete all MediaTag and PageElement using this location
+            MediaTag.objects.filter(location=location).delete()
+            elements = PageElement.objects.filter(text=location)
+            for element in elements:
+                element.text = ""
+                element.save()
+        return Response({
+            'detail': _('Media correctly deleted.')},
+            status=status.HTTP_200_OK)
 
     def put(self, request, *args, **kwargs):
         """
@@ -232,11 +226,12 @@ class MediaListAPIView(UploadedImageMixin, AccountMixin, ListCreateAPIView):
             parts = urlparse(item['location'])
             location = urlunparse((parts.scheme, parts.netloc, parts.path,
                 None, None, None))
-            media_tags = MediaTag.objects.filter(location=location)
-            for tag in tags:
-                MediaTag.objects.get_or_create(location=location, tag=tag)
-            # Remove tags which are no more set for the location.
-            media_tags.exclude(tag__in=tags).delete()
+            with transaction.atomic():
+                media_tags = MediaTag.objects.filter(location=location)
+                for tag in tags:
+                    MediaTag.objects.get_or_create(location=location, tag=tag)
+                # Remove tags which are no more set for the location.
+                media_tags.exclude(tag__in=tags).delete()
 
             # Update tags returned by the API.
             item['tags'] = ",".join(list(MediaTag.objects.filter(
@@ -244,4 +239,6 @@ class MediaListAPIView(UploadedImageMixin, AccountMixin, ListCreateAPIView):
 
         serializer = self.get_serializer(
             sorted(assets, key=lambda x: x['updated_at']), many=True)
-        return self.get_paginated_response(serializer.data)
+        http_resp = self.get_paginated_response(serializer.data)
+        http_resp.data.update({'detail': _("Tags correctly updated.")})
+        return http_resp
