@@ -1,4 +1,4 @@
-# Copyright (c) 2020 DjaoDjin inc.
+# Copyright (c) 2021 DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -24,14 +24,15 @@
 
 import logging
 
-from boto.exception import S3ResponseError
+import markdown
+from bs4 import BeautifulSoup
 from django.http import Http404
 from django.utils._os import safe_join
 from rest_framework.generics import get_object_or_404
 
 from . import settings
-from .compat import six, urlsplit
-from .models import MediaTag, PageElement
+from .compat import import_string, six, urlsplit
+from .models import MediaTag, PageElement, get_active_theme
 from .extras import AccountMixinBase
 
 
@@ -46,23 +47,44 @@ class TrailMixin(object):
     """
     Generate a trail of PageElement based on a path.
     """
+    URL_PATH_SEP = '/'
+    DB_PATH_SEP = '/'
+
+    @property
+    def element(self):
+        if not hasattr(self, '_element'):
+            path = self.path.strip(self.URL_PATH_SEP)
+            if not path:
+                self._element = None
+            else:
+                parts = path.split(self.URL_PATH_SEP)
+                self._element = get_object_or_404(
+                    PageElement.objects.all(), slug=parts[-1])
+        return self._element
 
     @property
     def path(self):
         if not hasattr(self, '_path'):
             self._path = self.kwargs.get('path', '')
-            if self._path and not self._path.startswith('/'):
-                self._path = '/' + self._path
+            if self._path and not self._path.startswith(self.URL_PATH_SEP):
+                self._path = self.URL_PATH_SEP + self._path
         return self._path
 
-    @staticmethod
-    def get_full_element_path(path):
+    @property
+    def full_path(self):
+        if not hasattr(self, '_full_path'):
+            self._full_path = self.DB_PATH_SEP.join(
+                [str(elem) for elem in self.get_full_element_path(self.path)])
+            if (self._full_path and
+                not self._full_path.startswith(self.DB_PATH_SEP)):
+                self._full_path = self.DB_PATH_SEP + self._full_path
+        return self._full_path
+
+    def get_full_element_path(self, path):
         if not path:
             return []
-        parts = path.split('/')
-        if not parts[0]:
-            parts.pop(0)
         results = []
+        parts = path.strip(self.URL_PATH_SEP).split(self.URL_PATH_SEP)
         if parts:
             element = get_object_or_404(
                 PageElement.objects.all(), slug=parts[-1])
@@ -79,36 +101,37 @@ class TrailMixin(object):
             results = candidates[0]
         return results
 
+    def get_url_kwargs(self, **kwargs):
+        url_kwargs = super(TrailMixin, self).get_url_kwargs(**kwargs)
+        if 'path' in self.kwargs:
+            url_kwargs.update({'path': self.kwargs.get('path')})
+        return url_kwargs
 
-class PageElementMixin(AccountMixin):
 
-    lookup_field = 'slug'
-    lookup_url_kwarg = 'slug'
+class PageElementMixin(object):
 
-    def get_queryset(self):
-        return PageElement.objects.filter(account=self.account)
+    URL_PATH_SEP = '/'
 
-    def get_object(self):
-        queryset = self.filter_queryset(self.get_queryset())
+    element_field = 'slug'
+    element_url_kwarg = 'slug'
 
-        lookup_value = None
-        lookup_url_kwarg = self.lookup_url_kwarg or self.lookup_field
-        if lookup_url_kwarg in self.kwargs:
-            lookup_value = self.kwargs[lookup_url_kwarg]
-        else:
-            parts = self.kwargs.get('path').split('/')
-            for part in reversed(parts):
-                if part:
-                    lookup_value = part
-                    break
-
-        filter_kwargs = {self.lookup_field: lookup_value}
-        obj = get_object_or_404(queryset, **filter_kwargs)
-
-        # May raise a permission denied
-        self.check_object_permissions(self.request, obj)
-
-        return obj
+    @property
+    def element(self):
+        if not hasattr(self, '_element'):
+            element_value = None
+            element_url_kwarg = self.element_url_kwarg or self.element_field
+            if element_url_kwarg in self.kwargs:
+                element_value = self.kwargs[element_url_kwarg]
+            else:
+                path = self.kwargs.get('path', '').strip(self.URL_PATH_SEP)
+                if not path:
+                    raise Http404()
+                parts = path.split(self.URL_PATH_SEP)
+                element_value = parts[-1]
+            filter_kwargs = {self.element_field: element_value}
+            self._element = get_object_or_404(
+                PageElement.objects.all(), **filter_kwargs)
+        return self._element
 
 
 class UploadedImageMixin(object):
@@ -129,6 +152,7 @@ class UploadedImageMixin(object):
         """
         Return a list of media from default storage
         """
+        #pylint:disable=too-many-locals
         results = []
         total_count = 0
         if prefix.startswith('/'):
@@ -165,7 +189,7 @@ class UploadedImageMixin(object):
             if storage.exists('.'):
                 LOGGER.exception(
                     "Unable to list objects in %s.", storage.__class__.__name__)
-        except S3ResponseError:
+        except storage.connection_response_error:
             LOGGER.exception(
                 "Unable to list objects in 's3://%s/%s/%s'.",
                 storage.bucket_name, storage.location, prefix)
@@ -184,13 +208,12 @@ class ThemePackageMixin(AccountMixin):
         if not hasattr(self, '_theme'):
             self._theme = self.kwargs.get(self.theme_url_kwarg)
             if not self._theme:
-                self._theme = settings.APP_NAME
+                self._theme = get_active_theme()
         return self._theme
 
     @staticmethod
     def get_templates_dir(theme):
         if isinstance(settings.THEME_DIR_CALLABLE, six.string_types):
-            from .compat import import_string
             settings.THEME_DIR_CALLABLE = import_string(
                 settings.THEME_DIR_CALLABLE)
         theme_dir = settings.THEME_DIR_CALLABLE(theme)
@@ -199,3 +222,48 @@ class ThemePackageMixin(AccountMixin):
     @staticmethod
     def get_statics_dir(theme):
         return safe_join(settings.PUBLIC_ROOT, theme, 'static')
+
+
+class UpdateEditableMixin(object):
+    """
+    Edit an element in a page.
+    """
+    @staticmethod
+    def insert_formatted(editable, new_text):
+        new_text = BeautifulSoup(new_text, 'html5lib')
+        for image in new_text.find_all('img'):
+            image['style'] = "max-width:100%"
+        if editable.name == 'div':
+            editable.clear()
+            editable.append(new_text)
+        else:
+            editable.string = "ERROR : Impossible to insert HTML into \
+                \"<%s></%s>\" element. It should be \"<div></div>\"." %\
+                (editable.name, editable.name)
+            editable['style'] = "color:red;"
+            # Prevent edition of error notification
+            editable['class'] = editable['class'].remove("editable")
+
+    @staticmethod
+    def insert_currency(editable, new_text):
+        amount = float(new_text)
+        editable.string = "$%.2f" % (amount/100)
+
+    @staticmethod
+    def insert_markdown(editable, new_text):
+        new_text = markdown.markdown(new_text,)
+        new_text = BeautifulSoup(new_text, 'html.parser')
+        for image in new_text.find_all('img'):
+            image['style'] = "max-width:100%"
+        editable.name = 'div'
+        editable.string = ''
+        children_done = []
+        for element in new_text.find_all():
+            if element.name != 'html' and\
+                element.name != 'body':
+                if element.findChildren():
+                    for sub_el in element.findChildren():
+                        element.append(sub_el)
+                        children_done += [sub_el]
+                if not element in children_done:
+                    editable.append(element)
