@@ -1,4 +1,4 @@
-# Copyright (c) 2020, Djaodjin Inc.
+# Copyright (c) 2021, Djaodjin Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -23,13 +23,15 @@
 # ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 from __future__ import unicode_literals
 
+import json
+
 import bleach
-from django.db import transaction
-from django.shortcuts import get_object_or_404
 from django.utils.translation import ugettext_lazy as _
 from rest_framework import serializers
 
-from .models import PageElement, ThemePackage, LessVariable
+from .compat import is_authenticated
+from .models import (Comment, Follow, LessVariable, PageElement, ThemePackage,
+    Vote)
 from .settings import ALLOWED_TAGS, ALLOWED_ATTRIBUTES, ALLOWED_STYLES
 
 #pylint: disable=no-init,abstract-method
@@ -77,7 +79,8 @@ class EdgeCreateSerializer(serializers.Serializer):
     external_key = serializers.CharField(required=False)
 
 
-class RelationShipSerializer(serializers.Serializer): #pylint: disable=abstract-method
+class RelationShipSerializer(serializers.Serializer):
+    #pylint: disable=abstract-method
     orig_elements = serializers.ListField(
         child=serializers.SlugField(), required=False
         )
@@ -85,57 +88,135 @@ class RelationShipSerializer(serializers.Serializer): #pylint: disable=abstract-
         child=serializers.SlugField(), required=False
         )
 
+
+class CommentSerializer(serializers.ModelSerializer):
+    """
+    Serializes a Comment.
+    """
+    text = HTMLField(html_tags=ALLOWED_TAGS, html_attributes=ALLOWED_ATTRIBUTES,
+        html_styles=ALLOWED_STYLES, required=False,
+        help_text=_("Long description of the page element"))
+    user = serializers.SlugRelatedField(read_only=True, slug_field='username')
+
+    class Meta:
+        model = Comment
+        fields = ('text', 'created_at', 'user')
+        read_only_fields = ('created_at', 'user')
+
+
+class NodeElementSerializer(serializers.ModelSerializer):
+    """
+    Serializes a PageElement as a node in a content tree
+    """
+    path = serializers.CharField(allow_null=True)
+    indent = serializers.SerializerMethodField()
+    account = serializers.SlugRelatedField(slug_field='slug',
+        read_only=True, required=False,
+        help_text=("Account that can edit the page element"))
+    picture = serializers.CharField(required=False, allow_null=True,
+        help_text=_("Picture icon that can be displayed alongside the title"))
+    extra = serializers.SerializerMethodField(required=False, allow_null=True,
+        help_text=_("Extra meta data (can be stringify JSON)"))
+
+    class Meta:
+        model = PageElement
+        fields = ('path', 'account', 'title', 'picture', 'indent', 'extra')
+        read_only_fields = ('slug',)
+
+    def get_extra(self, obj):
+        try:
+            return obj.get('extra', {})
+        except AttributeError:
+            pass
+        try:
+            return obj.extra
+        except AttributeError:
+            pass
+        return {}
+
+    def get_indent(self, obj):
+        try:
+            return obj.get('indent', 0)
+        except AttributeError:
+            pass
+        try:
+            return obj.indent
+        except AttributeError:
+            pass
+        return 0
+
+
 class PageElementSerializer(serializers.ModelSerializer):
     """
     Serializes a PageElement.
     """
 
+    path = serializers.CharField(required=False, allow_null=True)
     slug = serializers.SlugField(required=False,
         help_text=_("Unique identifier that can be used in URL paths"))
-    path = serializers.SerializerMethodField()
+    account = serializers.SlugRelatedField(slug_field='slug',
+        read_only=True, required=False,
+        help_text=("Account that can edit the page element"))
+    picture = serializers.CharField(required=False, allow_null=True,
+        help_text=_("Picture icon that can be displayed alongside the title"))
     text = HTMLField(html_tags=ALLOWED_TAGS, html_attributes=ALLOWED_ATTRIBUTES,
         html_styles=ALLOWED_STYLES, required=False,
         help_text=_("Long description of the page element"))
-    tag = serializers.CharField(required=False,
+    extra = serializers.SerializerMethodField(required=False, allow_null=True,
         help_text=_("Extra meta data (can be stringify JSON)"))
-    orig_elements = serializers.ListField(
-        child=serializers.SlugField(required=False), required=False
-        )
-    dest_elements = serializers.ListField(
-        child=serializers.SlugField(required=False), required=False
-        )
+    nb_upvotes = serializers.IntegerField(required=False)
+    nb_followers = serializers.IntegerField(required=False)
+    upvote = serializers.SerializerMethodField(required=False, allow_null=True)
+    follow = serializers.SerializerMethodField(required=False, allow_null=True)
+    count = serializers.IntegerField(required=False)
+    results = serializers.ListField(required=False,
+        child=NodeElementSerializer())
 
     class Meta:
         model = PageElement
-        fields = ('slug', 'path', 'title', 'text', 'tag',
-                  'orig_elements', 'dest_elements')
+        fields = ('path', 'slug', 'picture', 'title', 'text', 'reading_time',
+            'lang', 'account', 'extra',
+            'nb_upvotes', 'nb_followers', 'upvote', 'follow',
+            'count', 'results')
+        read_only_fields = ('path', 'slug', 'account',
+            'nb_upvotes', 'nb_followers', 'upvote', 'follow',
+            'count', 'results')
+
+    def get_extra(self, obj):
+        if not isinstance(obj.extra, dict):
+            try:
+                return json.loads(obj.extra)
+            except (TypeError, ValueError):
+                pass
+        return obj.extra
+
+    def get_upvote(self, data):
+        request = self.context.get('request')
+        if request and is_authenticated(request):
+            vote = Vote.objects.filter(
+                user=request.user, element=data).first()
+            return vote and vote.vote == Vote.UP_VOTE
+        return None
+
+    def get_follow(self, data):
+        request = self.context.get('request')
+        if request and is_authenticated(request):
+            follow = Follow.objects.filter(
+                user=request.user, element=data).first()
+            return follow is not None
+        return None
 
     def get_path(self, obj):
         prefix = self.context.get('prefix', "")
         if not prefix:
             parents = obj.get_parent_paths()
             if parents:
-                prefix = '/%s' % "/".join(
+                prefix = "/".join(
                     [parent.slug for parent in parents[0][:-1]])
+                if prefix:
+                    prefix = "/" + prefix
         return "%s/%s" % (prefix, obj.slug)
 
-    def create(self, validated_data):
-        orig_elements = validated_data.pop('orig_elements', None)
-        dest_elements = validated_data.pop('dest_elements', None)
-        with transaction.atomic():
-            instance = super(PageElementSerializer, self).create(validated_data)
-            if orig_elements:
-                for orig_element in orig_elements:
-                    orig_element = get_object_or_404(
-                        PageElement, slug=orig_element)
-                    orig_element.add_relationship(instance)
-            if dest_elements:
-                for dest_element in dest_elements:
-                    dest_element = get_object_or_404(
-                        PageElement, slug=dest_element)
-                    instance.add_relationship(dest_element)
-        return instance
-
 
 class PageElementTagSerializer(serializers.ModelSerializer):
 
@@ -144,24 +225,6 @@ class PageElementTagSerializer(serializers.ModelSerializer):
         fields = ('tag',)
 
 
-class NodeElementSerializer(serializers.ModelSerializer):
-    """
-    Serializes a PageElement as a node in a content tree
-    """
-    title = serializers.CharField()
-    picture = serializers.CharField(required=False, allow_null=True,
-        help_text=_("Picture icon that can be displayed alongside the title"))
-    extra = serializers.CharField(required=False, allow_null=True,
-        help_text=_("Extra meta data (can be stringify JSON)"))
-
-    class Meta:
-        model = PageElement
-        fields = ('title', 'picture', 'extra', 'children')
-
-NodeElementSerializer._declared_fields['children'] = NodeElementSerializer(
-    many=True)
-
-
 class LessVariableSerializer(serializers.ModelSerializer):
     class Meta:
         model = LessVariable
@@ -199,3 +262,21 @@ class MediaItemListSerializer(NoModelSerializer):
 
 class EditionFileSerializer(serializers.Serializer):
     text = serializers.CharField(allow_blank=True)
+
+
+class SourceCodeSerializer(NoModelSerializer):
+
+    path = serializers.CharField(required=False, max_length=255)
+    text = serializers.CharField(required=False, max_length=100000)
+
+
+class HintSerializer(NoModelSerializer):
+
+    index = serializers.IntegerField()
+    name = serializers.CharField()
+
+
+class SourceElementSerializer(SourceCodeSerializer):
+
+    hints = serializers.ListField(
+        child=HintSerializer(), required=False)
