@@ -114,7 +114,7 @@ class AssetsParser(Parser):
                         if hasattr(self, 'error'):
                             raise self.error(token, err)
                         # Django < 1.8
-                        elif not self.compile_function_error(token, err):
+                        if not self.compile_function_error(token, err):
                             raise
                 elif command == 'static':
                     self.dest_stream.write(
@@ -271,14 +271,16 @@ def install_theme_fileobj(theme_name, zip_file, force=False):
                     if tmp_path:
                         if not os.path.isdir(os.path.dirname(tmp_path)):
                             os.makedirs(os.path.dirname(tmp_path))
-                        template_string = zip_file.read(info.filename)
-                        if hasattr(template_string, 'decode'):
-                            template_string = template_string.decode('utf-8')
+                        template_bytes = zip_file.read(info.filename)
+                        if hasattr(template_bytes, 'decode'):
+                            template_string = template_bytes.decode('utf-8')
+                        else:
+                            template_string = template_bytes
                         template_string = force_text(template_string)
                         try:
                             check_template(template_string)
-                            with open(tmp_path, 'w') as extracted_file:
-                                extracted_file.write(template_string)
+                            with open(tmp_path, 'wb') as extracted_file:
+                                extracted_file.write(template_bytes)
                             try:
                                 default_path = get_template_path(
                                     relative_path=relative_path)
@@ -338,7 +340,7 @@ def remove_theme(theme_name):
     except NameError:
         # py27 `rmtree` will raise an OSError executing `os.listdir(path)`
         # if the path is not present.
-        FileNotFoundError = OSError
+        FileNotFoundError = OSError #pylint:disable=redefined-builtin
     theme_dir = get_theme_dir(theme_name)
     public_dir = safe_join(settings.PUBLIC_ROOT, theme_name)
     LOGGER.info("remove theme '%s', that is directories %s and %s.",
