@@ -1,4 +1,4 @@
-# Copyright (c) 2020, DjaoDjin inc.
+# Copyright (c) 2021, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -31,7 +31,7 @@ from django.core.validators import RegexValidator
 from django.utils.module_loading import import_string
 from django.utils.translation import ugettext_lazy as _
 
-from .compat import urljoin
+from .compat import six, urljoin
 
 
 LOGGER = logging.getLogger(__name__)
@@ -102,10 +102,11 @@ def get_default_storage_base(request, account=None, **kwargs):
         for key in ['access_key', 'secret_key', 'security_token']:
             if key in request.session:
                 storage_kwargs[key] = request.session[key]
-        return storage_class(
-            bucket=_get_bucket_name(account),
-            location=_get_media_prefix(account),
-            **storage_kwargs)
+        bucket = _get_bucket_name(account)
+        location = _get_media_prefix(account)
+        LOGGER.debug("create %s(bucket='%s', location='%s', %s)",
+            storage_class.__name__, bucket, location, storage_kwargs)
+        return storage_class(bucket=bucket, location=location, **storage_kwargs)
     except AttributeError:
         LOGGER.debug("``%s`` does not contain a ``bucket_name``"\
             " field, default to FileSystemStorage.", storage_class)
@@ -151,3 +152,20 @@ def _get_media_prefix(account=None):
         if not media_prefix:
             media_prefix = str(account)
     return media_prefix
+
+
+def update_context_urls(context, urls):
+    if 'urls' in context:
+        for key, val in six.iteritems(urls):
+            if key in context['urls']:
+                if isinstance(val, dict):
+                    context['urls'][key].update(val)
+                else:
+                    # Because organization_create url is added in this mixin
+                    # and in ``OrganizationRedirectView``.
+                    context['urls'][key] = val
+            else:
+                context['urls'].update({key: val})
+    else:
+        context.update({'urls': urls})
+    return context
